package cd.casic.ci.common.pipeline.matrix;
import cd.casic.ci.common.pipeline.utils.YamlUtil;
import cd.casic.ci.common.pipeline.utils.MatrixContextUtils;
import cn.hutool.core.lang.Pair;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.*;


/**
 * @author by mianbin
 * @Classname MatrixConfig
 * @Description TODO
 * @Date 2025/3/29 15:46
 */


@Data
@AllArgsConstructor
@NoArgsConstructor
@Schema(title = "矩阵的分裂计算配置")
public class MatrixConfig {
    @Schema(title = "分裂策略", required = true)
    private Map<String, List<String>> strategy;
    @Schema(title = "额外的参数组合", required = true)
    private List<Map<String, String>> include;
    @Schema(title = "排除的参数组合", required = false)
    private List<Map<String, String>> exclude;

    public static final String MATRIX_CONTEXT_KEY_PREFIX = "matrix.";

    /**
     * 根据[strategy], [include], [exclude]矩阵参数计算最终参数组合列表
     */
    public List<Map<String, String>> getAllCombinations() {
        List<Map<String, String>> combinations = new ArrayList<>();
        Pair<List<String>, List<Map<String, String>>> result = calculateContextMatrix(strategy);
        List<String> keyList = result.getKey();
        List<Map<String, String>> strategyCase = result.getValue();
        combinations.addAll(strategyCase);

        // 将额外添加的参数在匹配的组合内进行追加
        List<Map<String, String>> caseToAdd = new ArrayList<>();
        if (include != null) {
            for (Map<String, String> includeCase : include) {
                // 如果strategy为空，直接添加include
                if (strategyCase.isEmpty()) {
                    caseToAdd.add(new HashMap<>(includeCase));
                }
                // 筛选出所有与矩阵匹配的key
                List<String> matchKey = new ArrayList<>();
                for (String key : includeCase.keySet()) {
                    if (keyList.contains(key)) {
                        matchKey.add(key);
                    }
                }
                // 如果没有匹配的key则直接丢弃
                if (matchKey.isEmpty()) {
                    continue;
                }
                boolean expanded = false;
                List<Map<String, String>> caseToAddTmp = new ArrayList<>();
                for (Map<String, String> var : combinations) {
                    if (keyValueMatch(var, includeCase, matchKey)) {
                        // 将全匹配的额外参数直接追加到匹配的组合
                        var.putAll(includeCase);
                        expanded = true;
                    } else {
                        // 不能全匹配的额外参数作为一个新组合加入
                        caseToAddTmp.add(new HashMap<>(includeCase));
                    }
                }
                if (!expanded) {
                    caseToAdd.addAll(caseToAddTmp);
                }
            }
        }
        combinations.addAll(caseToAdd);
        // 计算strategy和include后，再进行组合排除
        if (exclude != null) {
            combinations.removeAll(exclude);
        }

        List<Map<String, String>> finalResult = new ArrayList<>();
        for (Map<String, String> contextCase : combinations) {
            // 临时方案：支持解析value中的一级对象访问
            Map<String, String> resultCase = new HashMap<>();
            for (Map.Entry<String, String> entry : contextCase.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                resultCase.put(MATRIX_CONTEXT_KEY_PREFIX + key, value);
                try {
                    Map<String, Object> map = YamlUtil.to(value, new com.fasterxml.jackson.core.type.TypeReference<Map<String, Object>>() {
                    });
                    if (map != null) {
                        for (Map.Entry<String, Object> pairEntry : map.entrySet()) {
                            String pair = pairEntry.getKey();
                            String[] split = pair.split("=");
                            if (split.length == 2) {
                                resultCase.put(MATRIX_CONTEXT_KEY_PREFIX + key + "." + split[0], split[1]);
                            }
                        }
                    }
                } catch (Exception e) {
                    // 忽略解析异常
                }
            }
            finalResult.add(resultCase);
        }
        // 去重
        List<Map<String, String>> distinctResult = new ArrayList<>();
        for (Map<String, String> map : finalResult) {
            if (!distinctResult.contains(map)) {
                distinctResult.add(map);
            }
        }
        return distinctResult;
    }

    /**
     * 根据[strategyMap]矩阵生成所有参数组合列表
     */
    private Pair<List<String>, List<Map<String, String>>> calculateContextMatrix(Map<String, List<String>> strategyMap) {
        if (strategyMap == null || strategyMap.isEmpty()) {
            return new Pair<>(Collections.emptyList(), Collections.emptyList());
        }
        List<Map<String, String>> caseList = new ArrayList<>();
        List<String> keyList = new ArrayList<>(strategyMap.keySet());
        ArrayList<List<String>> valueLists = new ArrayList<>(strategyMap.values());
        MatrixContextUtils.loopCartesianProduct(valueLists).forEach(valueList -> {
            Map<String, String> var = new HashMap<>();
            for (int i = 0; i < keyList.size(); i++) {
                var.put(keyList.get(i), valueList.get(i).toString());
            }
            caseList.add(var);
        });
        return new Pair<>(keyList, caseList);
    }

    /**
     * 对比[case]和[includeCase]中所有匹配key[matchKey]的值是否相同
     * 出现任意不同的情况则不是全匹配
     */
    private boolean keyValueMatch(Map<String, String> var, Map<String, String> includeCase, List<String> matchKey) {
        for (String key : matchKey) {
            if (!Objects.equals(var.get(key), includeCase.get(key))) {
                return false;
            }
        }
        return true;
    }
}