package cd.casic.ci.common.pipeline.pojo;

import cd.casic.ci.common.pipeline.dialect.IPipelineDialect;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.ManualReviewAction;
import cd.casic.ci.common.pipeline.option.StageControlOption;
import cd.casic.ci.common.pipeline.pojo.element.atom.ManualReviewParam;
import cd.casic.framework.commons.util.date.DateUtils;
import cn.hutool.core.util.IdUtil;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo
 * @Project：ops-pro
 * @name：StagePauseCheck
 * @Date：2025/03/25 16:34
 * @Filename：StagePauseCheck
 * @description：Todo
 */
@Data
@Builder
@Accessors(chain = true)
@AllArgsConstructor
@NoArgsConstructor
@Schema(title = "stage准入准出配置模型")
public class StagePauseCheck {
    @Schema(title = "是否人工触发", required = false)
    private Boolean manualTrigger = false;
    @Schema(title = "状态", required = false)
    private String status;
    @Schema(title = "审核说明", required = false)
    private String reviewDesc;
    @Schema(title = "审核流配置", required = false)
    private List<StageReviewGroup> reviewGroups;
    @Schema(title = "审核变量", required = false)
    private List<ManualReviewParam> reviewParams;
    @Schema(title = "等待审核的超时时间，默认24小时兜底", required = false)
    private Integer timeout = 24;
    @Schema(title = "质量红线规则ID集合", required = false)
    private List<String> ruleIds;
    @Schema(title = "记录本次构建质量红线规则的检查次数", required = false)
    private Integer checkTimes;
    @Schema(title = "是否以markdown格式发送审核说明", required = false)
    private Boolean markdownContent = false;
    @Schema(title = "发送的通知类型", required = false)
    private List<String> notifyType = List.of("RTX");
    @Schema(title = "企业微信群id", required = false)
    private List<String> notifyGroup;

    /**
     * 获取当前等待中的审核组
     */
    public StageReviewGroup groupToReview() {
        return reviewGroups.stream()
                .filter(group -> group.getStatus() == null || group.getStatus().isBlank())
                .findFirst()
                .orElse(null);
    }

    /**
     * 判断操作用户在不在当前审核人员名单中
     */
    public boolean reviewerContains(String userId) {
        return reviewGroups.stream()
                .anyMatch(group -> group.getStatus() == null && group.getReviewers().contains(userId));
    }

    /**
     * 审核通过当前等待中的审核组
     */
    public StageReviewGroup reviewGroup(String userId, ManualReviewAction action, String groupId, List<ManualReviewParam> params, String suggest) {
        StageReviewGroup group = getReviewGroupById(groupId);
        if (group != null && group.getStatus() == null) {
            group.setStatus(action.name());
            group.setOperator(userId);
            group.setReviewTime(DateUtils.LocalDateTimeToMilli(LocalDateTime.now()));
            group.setSuggest(suggest);
            group.setParams(parseReviewParams(params));
            if (groupToReview() == null) {
                status = BuildStatus.REVIEW_PROCESSED.name();
            } else if (action == ManualReviewAction.ABORT) {
                status = BuildStatus.REVIEW_ABORT.name();
            }
        }
        return group;
    }

    /**
     * 获取指定ID的审核组
     */
    public StageReviewGroup getReviewGroupById(String groupId) {
        if (groupId == null || groupId.isBlank()) {
            return reviewGroups.get(0);
        }
        return reviewGroups.stream()
                .filter(group -> groupId.equals(group.getId()))
                .findFirst()
                .orElse(null);
    }

    /**
     * 初始化状态并，填充审核组ID
     */
    public void fixReviewGroups(boolean init) {
        reviewGroups.forEach(group -> {
            if (group.getId() == null || group.getId().isBlank()) {
                group.setId(IdUtil.fastSimpleUUID());
            }
            if (init) {
                group.setStatus(null);
                group.setReviewTime(null);
                group.setOperator(null);
                group.setParams(null);
                group.setSuggest(null);
            }
        });
        if (init) {
            status = null;
            checkTimes = null;
        }
    }

    /**
     * 处理审核参数 - 与默认值相同的变量不显示
     */
    public List<ManualReviewParam> parseReviewParams(List<ManualReviewParam> params) {
        try {
            if (reviewParams == null || params == null) {
                return null;
            }
            Map<String, ManualReviewParam> originMap = reviewParams.stream()
                    .collect(Collectors.toMap(ManualReviewParam::getKey, param -> param));
            List<ManualReviewParam> diff = new ArrayList<>();
            for (ManualReviewParam param : params) {
                if (!originMap.containsKey(param.getKey())) {
                    continue;
                }
                if (!originMap.get(param.getKey()).getValue().equals(param.getValue())) {
                    diff.add(param);
                    originMap.get(param.getKey()).setValue(param.getValue());
                }
            }
            reviewParams.forEach(param -> param.setValue(originMap.get(param.getKey()).getValue()));
            return diff;
        } catch (Exception ignore) {
            return null;
        }
    }

    /**
     * 进入审核流程前完成所有审核人变量替换
     */
    public void parseReviewVariables(Map<String, String> variables, IPipelineDialect dialect) {
//        Map<String, String> contextPair = EnvReplacementParser.getCustomExecutionContextByMap(variables);
//        reviewGroups.forEach(group -> {
//            if (group.getStatus() != null) {
//                return;
//            }
//            if (!group.getReviewers().isEmpty()) {
//                String reviewers = String.join(",", group.getReviewers());
//                List<String> realReviewers = EnvReplacementParser.parse(reviewers, variables, dialect.supportUseExpression(), contextPair)
//                        .split(",")
//                        .toList();
//                group.setReviewers(realReviewers);
//            }
//            if (!group.getGroups().isEmpty()) {
//                String groups = String.join(",", group.getGroups());
//                List<String> realGroups = EnvReplacementParser.parse(groups, variables, dialect.supportUseExpression(), contextPair)
//                        .split(",")
//                        .toList();
//                group.setGroups(realGroups);
//            }
//        });
//        reviewDesc = EnvReplacementParser.parse(reviewDesc, variables, dialect.supportUseExpression(), contextPair);
//        notifyGroup = notifyGroup.stream()
//                .map(group -> EnvReplacementParser.parse(group, variables, dialect.supportUseExpression(), contextPair))
//                .collect(Collectors.toList());
//        reviewParams.forEach(param -> param.parseValueWithType(variables));
    }

    /**
     * 重新恢复所有准入/准出状态
     */
    public void retryRefresh() {
        status = null;
        reviewGroups.forEach(group -> {
            group.setStatus(null);
            group.setParams(null);
            group.setOperator(null);
            group.setReviewTime(null);
            group.setSuggest(null);
        });
    }

    public static StagePauseCheck convertControlOption(StageControlOption stageControlOption) {
        return new StagePauseCheck()
                .setManualTrigger(stageControlOption.getManualTrigger())
                .setStatus(stageControlOption.getTriggered() ? BuildStatus.REVIEW_PROCESSED.name() : null)
                .setReviewGroups(List.of(new StageReviewGroup()
                        .setId(IdUtil.fastUUID())
                        .setReviewers(stageControlOption.getTriggerUsers())
                        .setStatus(stageControlOption.getTriggered() ? ManualReviewAction.PROCESS.name() : null)
                        .setParams(stageControlOption.getReviewParams())))
                .setReviewDesc(stageControlOption.getReviewDesc())
                .setReviewParams(stageControlOption.getReviewParams())
                .setTimeout(stageControlOption.getTimeout());
    }
}
